import logging
from typing import Dict, Any, List
from prefect import task
import time
from pyspark.sql import DataFrame
from pyspark.sql.functions import when, lit, sum as py_sum, count, coalesce, expr
from graphframes import GraphFrame

from Utils.SparkUtils import createSession, find_related_vertices
from Enum.DatabaseTypeEnum import DatabaseTypeEnum

logger = logging.getLogger()


class CommissionProcess:

    def __init__(self, config: Dict[str, Any]) -> None:
        self.conf = config
        self.identifier = self.conf["identifier"]
        self.session = None
        self.period = self.conf["period"]

        self.inputs = {}
        self.inter_results = {}

    def init(self, spark_jars: Dict[str, str]) -> None:
        self.session = createSession(self.identifier, spark_jars, "")

    @task
    def get_inputs(self, db_type: DatabaseTypeEnum, starroks_jdbc: str, starroks_fe: str, redshift_url: str = "",
                   mysql_url: str = "") -> None:
        try:
            inputs = self.conf["inputs"]
            database = starroks_jdbc[starroks_jdbc.rfind("/") + 1:]

            for input_obj in inputs["data"]:
                identifier = input_obj["identifier"]
                if db_type == DatabaseTypeEnum.REDSHIFT:
                    redshift_user = self.conf["redshift"]["user"]
                    redshift_pass = self.conf["redshift"]["password"]
                    df_input = self.session.read \
                        .format("jdbc") \
                        .option("driver", "com.amazon.redshift.jdbc42.Driver") \
                        .option("url", redshift_url) \
                        .option("dbtable", identifier) \
                        .option("user", redshift_user) \
                        .option("password", redshift_pass) \
                        .load()
                elif db_type == DatabaseTypeEnum.MYSQL:
                    mysql_user = self.conf["mysql"]["user"]
                    mysql_pass = self.conf["mysql"]["password"]
                    df_input = self.session.read \
                      .format("jdbc") \
                      .option("driver", "com.mysql.cj.jdbc.Driver") \
                      .option("url", mysql_url) \
                      .option("dbtable",  identifier) \
                      .option("user", mysql_user) \
                      .option("password", mysql_pass) \
                      .load()
                else:
                    starroks_user = self.conf["starroks"]["user"]
                    starroks_pass = self.conf["starroks"]["password"]
                    df_input = self.session.read.format("starrocks"). \
                        option("starrocks.table.identifier", database+"."+identifier). \
                        option("starrocks.fe.http.url", starroks_fe). \
                        option("starrocks.fe.jdbc.url", starroks_jdbc). \
                        option("starrocks.user", starroks_user). \
                        option("starrocks.password", starroks_pass).load()
                self.inputs.update({identifier: df_input})
        except Exception as e:
            raise AssertionError(f"Error in function 'get_inputs'. {e}")

    @task
    def filter_data(self, identifier: str) -> None:
        try:
            this_df = self.inputs[identifier]
            this_df = this_df.filter(this_df["SERVICIO"] == "Postpaid")
            this_df = this_df.filter(this_df["FLAG_COMISIONABLE"] == "1")
            this_df = this_df.filter(this_df["ACTIVE_USER_TRAFFIC"] == "1")
            self.inputs[identifier] = this_df
        except Exception as e:
            raise AssertionError(f"Error filtrando datos. {e}")

    @task
    def create_jerarquia(self, identifiers: List[str], df_goals: DataFrame, df_executes: DataFrame, df_base: DataFrame):
        graph = None
        try:
            merged = df_goals.join(df_executes, ["AGENTE_COMISIONA"], 'left')
            merged = merged.join(df_base, ["AGENTE_COMISIONA"], 'left')

            agent_df = self.inputs[identifiers[0]]
            structure_df = self.inputs[identifiers[1]]
            uo_df = self.inputs[identifiers[2]]
            org_df = self.inputs[identifiers[3]]

            agent_1 = agent_df.join(structure_df, agent_df["PIIN_IDENT"] == structure_df["PIOS_INDID"], 'leftsemi')
            agent_1 = agent_1.withColumnRenamed("PIIN_IDENT", "id")
            agent_1 = agent_1.join(merged, agent_1["id"] == merged["AGENTE_COMISIONA"], "left")

            estructura_1 = structure_df.join(uo_df.select("PIOU_ORGID", "PIOU_RESPO"),
                                             structure_df["PIOS_ORGID"] == uo_df["PIOU_ORGID"], 'left')
            estructura_1 = estructura_1.withColumn("PIOS_SUPER",
                                                   coalesce(estructura_1["PIOS_SUPER"], estructura_1["PIOU_RESPO"]))
            estructura_2 = estructura_1.join(org_df.select("PIOR_ORGID", "PIOR_RESPO"),
                                             estructura_1["PIOS_ORGID"] == org_df["PIOR_ORGID"], 'left')
            estructura_2 = estructura_2.withColumn("PIOS_SUPER",
                                                   coalesce(estructura_2["PIOS_SUPER"], estructura_2["PIOR_RESPO"]))
            estructura_2 = estructura_2.drop(*["PIOU_ORGID", "PIOU_RESPO", "PIOR_ORGID", "PIOR_RESPO"])

            estructura_2 = estructura_2.withColumn("PIOS_RELATION", lit("responsable"))
            estructura_2 = estructura_2.select("PIOS_SUPER", "PIOS_INDID", "PIOS_RELATION")
            estructura_2 = estructura_2.withColumnRenamed("PIOS_SUPER", "src").withColumnRenamed("PIOS_INDID", "dst")
            graph = GraphFrame(agent_1, estructura_2)
        except Exception as e:
            logger.error(f"Error creando dataframe de jerarquia. {e}")
        finally:
            return graph

    @task
    def get_goals(self, gross_name: str, goals_name: str) -> DataFrame:
        result = None
        try:
            goal_df = self.inputs[goals_name]
            gross_df = self.inputs[gross_name]

            goal_df = goal_df.withColumn("META_1", when(goal_df["KPI"] == "EQUIPOS", goal_df["META_FINAL"]).otherwise(lit(0)))
            goal_df = goal_df.withColumn("META_2", when(goal_df["KPI"] == "GROSS B2B", goal_df["META_FINAL"]).otherwise(lit(0)))
            goal_df = goal_df.withColumn("META_3", when(goal_df["KPI"] == "GROSS POSPAGO", goal_df["META_FINAL"]).otherwise(lit(0)))

            sums = [py_sum(column).alias(column) for column in ["META_1", "META_2", "META_3"]]
            goal_group = goal_df.groupBy("CEDULA", "PERIODO_PROCESO_CODIGO").agg(*sums)

            gross_df_agent = gross_df.groupBy("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO").count(). \
                select("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO")
            goal_group = goal_group.withColumnRenamed("PERIODO_PROCESO_CODIGO", "PERIODO_PROCESO_CODIGO_2")
            condition = [gross_df_agent["AGENTE_COMISIONA"] == goal_group["CEDULA"],
                         gross_df_agent["PERIODO_PROCESO_CODIGO"] == goal_group["PERIODO_PROCESO_CODIGO_2"]]

            result = gross_df_agent.join(goal_group, condition, how='left')
            result = result.na.fill(value=0, subset=["META_1", "META_2", "META_3"])
            result = result.drop(*["CEDULA", "PERIODO_PROCESO_CODIGO_2"])
        except Exception as e:
            raise AssertionError(f"Error obteniendo metas. Segunda forma. {e}")
        finally:
            return result

    @task
    def get_goals_2(self, goals_name: str, structure_name: str) -> DataFrame:
        result = None
        try:
            goal_df = self.inputs[goals_name]
            structure_df = self.inputs[structure_name]

            goal_df = goal_df[goal_df["PERIODO_PROCESO_CODIGO"] == self.period]
            goal_df = goal_df.withColumn("META_1", when(goal_df["KPI"] == "EQUIPOS", goal_df["META_FINAL"]).otherwise(lit(0)))
            goal_df = goal_df.withColumn("META_2", when(goal_df["KPI"] == "GROSS B2B", goal_df["META_FINAL"]).otherwise(lit(0)))
            goal_df = goal_df.withColumn("META_3", when(goal_df["KPI"] == "GROSS POSPAGO", goal_df["META_FINAL"]).otherwise(lit(0)))

            sums = [py_sum(column).alias(column) for column in ["META_1", "META_2", "META_3"]]
            goal_group = goal_df.groupBy("CEDULA").agg(*sums)

            condition = [structure_df["PIOS_INDID"] == goal_group["CEDULA"]]

            result = structure_df.select("PIOS_INDID").join(goal_group, condition, how='left')
            result = result.na.fill(value=0, subset=["META_1", "META_2", "META_3"])
            result = result.drop(*["CEDULA"])
            result = result.withColumn("PERIODO_PROCESO_CODIGO", lit(self.period))
            result = result.withColumnRenamed("PIOS_INDID", "AGENTE_COMISIONA")
        except Exception as e:
            raise AssertionError(f"Error obteniendo metas. Segunda forma. {e}")
        finally:
            return result

    @task
    def get_executed(self, gross_name: str, team_name: str) -> DataFrame:
        result = None
        try:
            team_df = self.inputs[team_name]
            gross_df = self.inputs[gross_name]

            executed1 = team_df[team_df["MODELO_TIPO"] == "SMARTPHONE"]
            executed1 = executed1.groupBy("CONSULTOR_DOCUMENTO", "PERIODO_PROCESO_CODIGO").agg(count("MODELO_TIPO").
                                                                                               alias("EJECUTADO_1"))
            gross_df_agent = gross_df.groupBy("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO").count(). \
                select("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO")
            executed1 = executed1.withColumnRenamed("PERIODO_PROCESO_CODIGO", "PERIODO_PROCESO_CODIGO_2")
            condition = [gross_df_agent["AGENTE_COMISIONA"] == executed1["CONSULTOR_DOCUMENTO"],
                         gross_df_agent["PERIODO_PROCESO_CODIGO"] == executed1["PERIODO_PROCESO_CODIGO_2"]]
            result_1 = gross_df_agent.join(executed1, condition, how='left')
            result_1 = result_1.na.fill(value=0, subset=["EJECUTADO_1"])
            result_1 = result_1.drop(*["CONSULTOR_DOCUMENTO", "PERIODO_PROCESO_CODIGO_2"])

            gross_b2b = gross_df[(gross_df["SEGMENTO"] == "B2B") & (gross_df["SERVICIO"] == "Postpaid") &
                                 (gross_df["FLAG_COMISIONABLE"] == "1") & (gross_df["ACTIVE_USER_TRAFFIC"] == "1")]
            gross_b2b = gross_b2b.groupBy("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO").agg(count("SERVICIO").
                                                                                            alias("EJECUTADO_2"))
            gross_b2b = gross_b2b.withColumnRenamed("PERIODO_PROCESO_CODIGO", "PERIODO_PROCESO_CODIGO_2")
            gross_b2b = gross_b2b.withColumnRenamed("AGENTE_COMISIONA", "AGENTE_COMISIONA_2")
            condition = [gross_df_agent["AGENTE_COMISIONA"] == gross_b2b["AGENTE_COMISIONA_2"],
                         gross_df_agent["PERIODO_PROCESO_CODIGO"] == gross_b2b["PERIODO_PROCESO_CODIGO_2"]]
            result_2 = gross_df_agent.join(gross_b2b, condition, how='left')
            result_2 = result_2.na.fill(value=0, subset=["EJECUTADO_2"])
            result_2 = result_2.drop(*["AGENTE_COMISIONA_2", "PERIODO_PROCESO_CODIGO_2"])

            gross_b2c = gross_df[(gross_df["SEGMENTO"] == "B2C") & (gross_df["SERVICIO"] == "Postpaid") &
                                 (gross_df["FLAG_COMISIONABLE"] == "1") & (gross_df["ACTIVE_USER_TRAFFIC"] == "1")]
            gross_b2c = gross_b2c.groupBy("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO").agg(count("SERVICIO").
                                                                                            alias("EJECUTADO_3"))
            gross_b2c = gross_b2c.withColumnRenamed("PERIODO_PROCESO_CODIGO", "PERIODO_PROCESO_CODIGO_2")
            gross_b2c = gross_b2c.withColumnRenamed("AGENTE_COMISIONA", "AGENTE_COMISIONA_2")
            condition = [gross_df_agent["AGENTE_COMISIONA"] == gross_b2c["AGENTE_COMISIONA_2"],
                         gross_df_agent["PERIODO_PROCESO_CODIGO"] == gross_b2c["PERIODO_PROCESO_CODIGO_2"]]
            result_3 = gross_df_agent.join(gross_b2c, condition, how='left')
            result_3 = result_3.na.fill(value=0, subset=["EJECUTADO_3"])
            result_3 = result_3.drop(*["AGENTE_COMISIONA_2", "PERIODO_PROCESO_CODIGO_2"])

            result = result_1.join(result_2, ["AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO"], 'left')
            result = result.join(result_3, ["AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO"])
        except Exception as e:
            logger.error(f"Error obteniendo ejecutados. Segunda forma. {e}")
        finally:
            return result

    @task
    def get_executed_2(self, structure_name: str, team_name: str, gross_name: str) -> DataFrame:
        result = None
        try:
            team_df = self.inputs[team_name]
            structure_df = self.inputs[structure_name]
            gross_df = self.inputs[gross_name]

            team_df = team_df[team_df["PERIODO_PROCESO_CODIGO"] == self.period]
            gross_df = gross_df[gross_df["PERIODO_PROCESO_CODIGO"] == self.period]
            executed1 = team_df[team_df["MODELO_TIPO"] == "SMARTPHONE"]
            executed1 = executed1.groupBy("CONSULTOR_DOCUMENTO").agg(count("MODELO_TIPO").alias("EJECUTADO_1"))

            condition = [structure_df["PIOS_INDID"] == executed1["CONSULTOR_DOCUMENTO"]]
            result_1 = structure_df.select("PIOS_INDID").join(executed1, condition, how='left')
            result_1 = result_1.na.fill(value=0, subset=["EJECUTADO_1"])
            result_1 = result_1.drop(*["CONSULTOR_DOCUMENTO"])

            gross_b2b = gross_df[(gross_df["SEGMENTO"] == "B2B") & (gross_df["SERVICIO"] == "Postpaid") &
                                 (gross_df["FLAG_COMISIONABLE"] == "1") & (gross_df["ACTIVE_USER_TRAFFIC"] == "1")]
            gross_b2b = gross_b2b.groupBy("AGENTE_COMISIONA").agg(count("SERVICIO").alias("EJECUTADO_2"))

            condition = [structure_df["PIOS_INDID"] == gross_b2b["AGENTE_COMISIONA"]]
            result_2 = structure_df.select("PIOS_INDID").join(gross_b2b, condition, how='left')
            result_2 = result_2.na.fill(value=0, subset=["EJECUTADO_2"])
            result_2 = result_2.drop(*["AGENTE_COMISIONA"])

            gross_b2c = gross_df[(gross_df["SEGMENTO"] == "B2C") & (gross_df["SERVICIO"] == "Postpaid") &
                                 (gross_df["FLAG_COMISIONABLE"] == "1") & (gross_df["ACTIVE_USER_TRAFFIC"] == "1")]
            gross_b2c = gross_b2c.groupBy("AGENTE_COMISIONA").agg(count("SERVICIO").alias("EJECUTADO_3"))

            # CONVERTIDO A SPARK SQL:

            # gross_df.createOrReplaceTempView("gross_df")
            #
            # # Escribir la consulta SQL
            # query = """
            #     SELECT AGENTE_COMISIONA, COUNT(SERVICIO) AS EJECUTADO_3
            #     FROM gross_df
            #     WHERE SEGMENTO = 'B2C' AND SERVICIO = 'Postpaid' AND FLAG_COMISIONABLE = '1' AND ACTIVE_USER_TRAFFIC = '1'
            #     GROUP BY AGENTE_COMISIONA
            # """
            #
            # # Ejecutar la consulta SQL
            # gross_b2c = spark.sql(query)

            condition = [structure_df["PIOS_INDID"] == gross_b2c["AGENTE_COMISIONA"]]
            result_3 = structure_df.select("PIOS_INDID").join(gross_b2c, condition, how='left')
            result_3 = result_3.na.fill(value=0, subset=["EJECUTADO_3"])
            result_3 = result_3.drop(*["AGENTE_COMISIONA"])

            result = result_1.join(result_2, ["PIOS_INDID"], 'left')
            result = result.join(result_3, ["PIOS_INDID"])
            result = result.withColumnRenamed("PIOS_INDID", "AGENTE_COMISIONA")
        except Exception as e:
            logger.error(f"Error obteniendo ejecutados. Segunda forma. {e}")
        finally:
            return result

    @task
    def update_executes(self, graph: GraphFrame, df_goals: DataFrame, df_executes: DataFrame, df_base: DataFrame):
        merged = None
        try:
            start_process = time.time()
            graph_related = find_related_vertices(graph)

            merged = df_goals.join(df_executes, "AGENTE_COMISIONA", 'left')
            merged = merged.join(df_base, "AGENTE_COMISIONA", 'left')

            graph_related_df = self.session.createDataFrame([(k, v) for k, v in graph_related.items()], ["vertex", "related_agents"])

            # Agregar los valores ejecutados por cada agente
            agg_df = merged.join(graph_related_df, expr("array_contains(related_agents, AGENTE_COMISIONA)"), "inner") \
                .groupBy("vertex") \
                .agg(py_sum("EJECUTADO_1").alias("sum_EJECUTADO_1"),
                     py_sum("EJECUTADO_2").alias("sum_EJECUTADO_2"),
                     py_sum("EJECUTADO_3").alias("sum_EJECUTADO_3"))

            merged = merged.join(agg_df, merged["AGENTE_COMISIONA"] == agg_df["vertex"], 'left')
            merged = merged.withColumn("EJECUTADO_1", when(merged["vertex"].isNotNull(), merged["sum_EJECUTADO_1"]).otherwise(merged["EJECUTADO_1"]))
            merged = merged.withColumn("EJECUTADO_2", when(merged["vertex"].isNotNull(), merged["sum_EJECUTADO_2"]).otherwise(merged["EJECUTADO_2"]))
            merged = merged.withColumn("EJECUTADO_3", when(merged["vertex"].isNotNull(), merged["sum_EJECUTADO_3"]).otherwise(merged["EJECUTADO_3"]))
            merged = merged.drop("vertex", "sum_EJECUTADO_1", "sum_EJECUTADO_2", "sum_EJECUTADO_3")
            print(f"Duración de creación de dataframes con grafos (jerarquía): {time.time() - start_process}")
        except Exception as e:
            logger.error(f"Error actualizando ejecutados por jerarquía. {e}")
        finally:
            return merged

    @task
    def get_source_value(self, gross_name: str, base_name: str) -> DataFrame:
        result = None
        try:
            base_df = self.inputs[base_name]
            gross_df = self.inputs[gross_name]

            base_df = base_df.select("PERIODO_PROCESO_CODIGO", "CEDULA", "VARIABLE_COMISION")
            base_df = base_df.dropDuplicates()

            gross_df_agent = gross_df.groupBy("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO").count(). \
                select("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO")
            base_df = base_df.withColumnRenamed("PERIODO_PROCESO_CODIGO", "PERIODO_PROCESO_CODIGO_2")
            condition = [gross_df_agent["AGENTE_COMISIONA"] == base_df["CEDULA"],
                         gross_df_agent["PERIODO_PROCESO_CODIGO"] == base_df["PERIODO_PROCESO_CODIGO_2"]]
            result = gross_df_agent.join(base_df, condition, how='left')
            result = result.na.fill(value=0, subset=["VARIABLE_COMISION"])
            result = result.drop(*["CEDULA", "PERIODO_PROCESO_CODIGO_2"])
        except Exception as e:
            logger.error(f"Error obteniendo valor de monto de origen. {e}")
        finally:
            return result

    @task
    def get_source_value_2(self, structure_name: str, base_name: str) -> DataFrame:
        result = None
        try:
            base_df = self.inputs[base_name]
            structure_df = self.inputs[structure_name]

            base_df = base_df[base_df["PERIODO_PROCESO_CODIGO"] == self.period]
            base_df = base_df.select("CEDULA", "VARIABLE_COMISION")
            base_df = base_df.dropDuplicates()

            condition = [structure_df["PIOS_INDID"] == base_df["CEDULA"]]
            result = structure_df.select("PIOS_INDID").join(base_df, condition, how='left')
            result = result.na.fill(value=0, subset=["VARIABLE_COMISION"])
            result = result.drop(*["CEDULA"])

            # Convertido a SQL

            # base_df.createOrReplaceTempView("base_df")
            # structure_df.createOrReplaceTempView("structure_df")
            #
            # query = """
            #     SELECT DISTINCT b.CEDULA, b.VARIABLE_COMISION
            #     FROM base_df b
            #     JOIN structure_df s ON s.PIOS_INDID = b.CEDULA
            #     WHERE b.PERIODO_PROCESO_CODIGO = '{}'
            # """.format(self.period)
            #
            # result = spark.sql(query)
            # result = result.na.fill(value=0, subset=["VARIABLE_COMISION"])
            # result = result.drop("CEDULA")

            result = result.withColumnRenamed("PIOS_INDID", "AGENTE_COMISIONA")
        except Exception as e:
            logger.error(f"Error obteniendo valor de monto de origen. {e}")
        finally:
            return result

    @task
    def get_commission_per_agent(self, df_goals: DataFrame, df_executes: DataFrame, df_base: DataFrame) -> DataFrame:
        try:
            merged = df_goals.join(df_executes, ["AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO"], 'left')
            merged = merged.join(df_base, ["AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO"], 'left')
            merged = merged.withColumn("CUMPLIMIENTO EQUIPOS",
                                       when(merged["META_1"] != 0, merged["EJECUTADO_1"]/merged["META_1"]).
                                       otherwise(lit(0)))
            merged = merged.withColumn("PONDERADO EQUIPOS", merged["CUMPLIMIENTO EQUIPOS"] * 0.25)
            merged = merged.withColumn("CUMPLIMIENTO B2B",
                                       when(merged["META_2"] != 0, merged["EJECUTADO_2"]/merged["META_2"]).
                                       otherwise(lit(0)))
            merged = merged.withColumn("PONDERADO B2B", merged["CUMPLIMIENTO B2B"] * 0.1)
            merged = merged.withColumn("CUMPLIMIENTO B2C",
                                       when(merged["META_3"] != 0, merged["EJECUTADO_3"]/merged["META_3"]).
                                       otherwise(lit(0)))
            merged = merged.withColumn("PONDERADO B2C", merged["CUMPLIMIENTO B2C"] * 0.65)
            merged = merged.withColumn("PONDERADO TOTAL", merged["PONDERADO EQUIPOS"] + merged["PONDERADO B2B"] + merged["PONDERADO B2C"])
            merged = merged.withColumn("FACTOR", when(merged["PONDERADO TOTAL"] <= 0.79, lit(0)).
                                       when((merged["PONDERADO TOTAL"] >= 0.8) & (merged["PONDERADO TOTAL"] < 2), merged["PONDERADO TOTAL"]).
                                       when(merged["PONDERADO TOTAL"] >= 2, lit(2)))
            merged = merged.withColumn("COMISION", merged["FACTOR"]*merged["VARIABLE_COMISION"])

            merged = merged.withColumnRenamed("EJECUTADO_2", "EJECUTADO B2B")
            merged = merged.withColumnRenamed("EJECUTADO_3", "EJECUTADO B2C")
            merged = merged.withColumnRenamed("EJECUTADO_1", "EJECUTADO Cant Smart")
            merged = merged.withColumnRenamed("META_1", "META EQUIPOS")
            merged = merged.withColumnRenamed("META_2", "META GROSS B2B")
            merged = merged.withColumnRenamed("META_3", "META GROSS POSPAGO")
            merged = merged.withColumnRenamed("VARIABLE_COMISION", "VARIABLE COMISION (monto de origen)")
        except Exception as e:
            logger.error(f"Error operando la comisión. {e}")
        finally:
            return

    @task
    def get_commission_per_agent_2(self, merged: DataFrame) -> DataFrame:
        try:
            merged = merged.withColumn("CUMPLIMIENTO EQUIPOS",
                                       when(merged["META_1"] != 0, merged["EJECUTADO_1"]/merged["META_1"]).
                                       otherwise(lit(0)))
            merged = merged.withColumn("PONDERADO EQUIPOS", merged["CUMPLIMIENTO EQUIPOS"] * 0.25)
            merged = merged.withColumn("CUMPLIMIENTO B2B",
                                       when(merged["META_2"] != 0, merged["EJECUTADO_2"]/merged["META_2"]).
                                       otherwise(lit(0)))
            merged = merged.withColumn("PONDERADO B2B", merged["CUMPLIMIENTO B2B"] * 0.1)
            merged = merged.withColumn("CUMPLIMIENTO B2C",
                                       when(merged["META_3"] != 0, merged["EJECUTADO_3"]/merged["META_3"]).
                                       otherwise(lit(0)))
            merged = merged.withColumn("PONDERADO B2C", merged["CUMPLIMIENTO B2C"] * 0.65)
            merged = merged.withColumn("PONDERADO TOTAL", merged["PONDERADO EQUIPOS"] + merged["PONDERADO B2B"] + merged["PONDERADO B2C"])
            merged = merged.withColumn("FACTOR", when(merged["PONDERADO TOTAL"] <= 0.79, lit(0)).
                                       when((merged["PONDERADO TOTAL"] >= 0.8) & (merged["PONDERADO TOTAL"] < 2), merged["PONDERADO TOTAL"]).
                                       when(merged["PONDERADO TOTAL"] >= 2, lit(2)))
            merged = merged.withColumn("COMISION", merged["FACTOR"]*merged["VARIABLE_COMISION"])

            merged = merged.withColumnRenamed("EJECUTADO_2", "EJECUTADO B2B")
            merged = merged.withColumnRenamed("EJECUTADO_3", "EJECUTADO B2C")
            merged = merged.withColumnRenamed("EJECUTADO_1", "EJECUTADO Cant Smart")
            merged = merged.withColumnRenamed("META_1", "META EQUIPOS")
            merged = merged.withColumnRenamed("META_2", "META GROSS B2B")
            merged = merged.withColumnRenamed("META_3", "META GROSS POSPAGO")
            merged = merged.withColumnRenamed("VARIABLE_COMISION", "VARIABLE COMISION (monto de origen)")
        except Exception as e:
            logger.error(f"Error operando la comisión. {e}")
        finally:
            return merged

    @task
    def write_result(self, df: DataFrame, table_name: str, db_type: DatabaseTypeEnum, starroks_jdbc: str, starroks_fe: str,
                     redshift_url: str = "", mysql_url: str = "") -> None:
        try:
            if db_type == DatabaseTypeEnum.REDSHIFT:
                df.write \
                    .format("jdbc") \
                    .option("driver", "com.amazon.redshift.jdbc42.Driver") \
                    .option("url", redshift_url) \
                    .option("dbtable", table_name) \
                    .option("user", "awsuser") \
                    .option("password", "Awsuser123") \
                    .mode("append") \
                    .save()
            elif db_type == DatabaseTypeEnum.MYSQL:
                mysql_user = self.conf["mysql"]["user"]
                mysql_pass = self.conf["mysql"]["password"]
                df.write \
                  .format("jdbc") \
                  .option("driver", "com.mysql.cj.jdbc.Driver") \
                  .option("url", mysql_url) \
                  .option("dbtable", table_name) \
                  .option("user", mysql_user) \
                  .option("password", mysql_pass) \
                  .mode("overwrite") \
                  .save()
            else:
                database = starroks_jdbc[starroks_jdbc.rfind("/") + 1:]
                starroks_user = self.conf["starroks"]["user"]
                starroks_pass = self.conf["starroks"]["password"]

                df.write.format("starrocks") \
                    .option("starrocks.fe.http.url", starroks_fe) \
                    .option("starrocks.fe.jdbc.url", starroks_jdbc) \
                    .option("starrocks.table.identifier", database+"."+table_name) \
                    .option("starrocks.user", starroks_user) \
                    .option("starrocks.password", starroks_pass) \
                    .mode("append") \
                    .save()
        except Exception as e:
            logger.error(f"Error guardando datos en BD. {e}")
