import time
import json
from typing import Any, Dict
from prefect import flow, get_run_logger


from Enum.DatabaseTypeEnum import DatabaseTypeEnum
from Enum.InputTypeEnum import InputTypeEnum
from Pipeline.ETLProcess import ETLProcess


SPARK_JARS = {
    "AWS_CORE": "/opt/spark-jars/hadoop-aws-3.3.4.jar",
    "BUNDLE": "/opt/spark-jars/aws-java-sdk-bundle-1.12.431.jar",
    "COMMON": "/opt/spark-jars/hadoop-common-3.3.4.jar",
    "AWS_CLIENT": "/opt/spark-jars/hadoop-client-3.3.4.jar",
    "STARROK": "/opt/spark-jars/starrocks-spark-connector-3.2_2.12-1.1.2.jar",
    "MYSQL": "/opt/spark-jars/mysql-connector-java-8.0.30.jar",
    "REDSHIFT": "/opt/spark-jars/redshift-jdbc42-2.1.0.12.jar"
}

STARROK_JDBC = "jdbc:mysql://192.168.1.37:9030/bcom_spark"
STARROK_FE_NODE = "192.168.1.37:8030"

REDSHIFT_JDBC = "jdbc:redshift://redshift-cluster-1.cumpswji5bs3.us-east-1.redshift.amazonaws.com:5439/dev?currentSchema=prueba_ca"

MYSQL_JDBC = "jdbc:mysql://localhost:13306/bcom_spark"

DB_TYPE = DatabaseTypeEnum.MYSQL

SOURCE_TYPE = InputTypeEnum.BUCKET


@flow
def run_etl(config: Dict[str, Any]) -> None:
    logger = get_run_logger()

    start_time = time.time()

    etl_process = ETLProcess(config)

    # Conexion a Spark (LocalMode, StandAlone or Clúster)
    start_init = time.time()
    etl_process.init(SPARK_JARS, SOURCE_TYPE)
    logger.info(f"Duración de creación de sesión Spark: {time.time() - start_init}")

    # Primer task - (Reader) - Extraer los ficheros
    start_reader = time.time()
    etl_process.reader(etl_process)
    logger.info(f"Duración de extracción de ficheros desde S3: {time.time() - start_reader}")

    # Segundo task - Setear esquema a las tablas
    start_transform = time.time()
    etl_process.set_schema(etl_process)

    # Process - Insumo Gross (Ventas)
    ventas_flag = etl_process.process_gross(etl_process, "VENTAS")

    # Process - Insumo Team (Equipos)
    teams_flag = etl_process.process_teams(etl_process, "DEVICES")
    logger.info(f"Duración de transformación y limpieza de datos: {time.time() - start_transform}")

    # Write - Insumo GROSS
    start_load = time.time()
    etl_process.write(etl_process, "VENTAS", STARROK_JDBC, STARROK_FE_NODE, ventas_flag, DB_TYPE,
                             REDSHIFT_JDBC, MYSQL_JDBC)
    # Write - Insumo DEVICES
    etl_process.write(etl_process, "DEVICES", STARROK_JDBC, STARROK_FE_NODE, teams_flag, DB_TYPE,
                             REDSHIFT_JDBC, MYSQL_JDBC)
    # Write - Insumo GOALS
    etl_process.write(etl_process, "GOALS", STARROK_JDBC, STARROK_FE_NODE, db_type=DB_TYPE,
                             redshift_url=REDSHIFT_JDBC, mysql_url=MYSQL_JDBC)
    # Write - Insumo PLANTA
    etl_process.write(etl_process, "COMERCIAL_BASE", STARROK_JDBC, STARROK_FE_NODE, db_type=DB_TYPE,
                             redshift_url=REDSHIFT_JDBC, mysql_url=MYSQL_JDBC)
    # Write - Insumo INDIVIDUOS
    etl_process.write(etl_process, "INDIVIDUOS", STARROK_JDBC, STARROK_FE_NODE, db_type=DB_TYPE,
                             redshift_url=REDSHIFT_JDBC, mysql_url=MYSQL_JDBC)
    # Write - Insumo ROLES
    etl_process.write(etl_process, "ROLES", STARROK_JDBC, STARROK_FE_NODE, db_type=DB_TYPE,
                             redshift_url=REDSHIFT_JDBC, mysql_url=MYSQL_JDBC)
    # Write - Insumo ORGANIZACION
    etl_process.write(etl_process, "ORGANIZACION", STARROK_JDBC, STARROK_FE_NODE, db_type=DB_TYPE,
                             redshift_url=REDSHIFT_JDBC, mysql_url=MYSQL_JDBC)
    # Write - Insumo UNIDADES
    etl_process.write(etl_process, "UNIDAD", STARROK_JDBC, STARROK_FE_NODE, db_type=DB_TYPE,
                             redshift_url=REDSHIFT_JDBC, mysql_url=MYSQL_JDBC)
    # Write - Insumo ESTRUCTURA
    etl_process.write(etl_process, "ESTRUCTURA_ORGANIZACIONAL", STARROK_JDBC, STARROK_FE_NODE, db_type=DB_TYPE,
                             redshift_url=REDSHIFT_JDBC, mysql_url=MYSQL_JDBC)
    logger.info(f"Duración de carga de datos a la BD: {time.time() - start_load}")

    logger.info(f"Duración de ejecución del proceso ETL General: {time.time() - start_time}")


if __name__ == "__main__":

    conf_path = "config.json"
    with open(conf_path) as f:
        conf = json.load(f)

    # Run ETL
    run_etl(conf)

